@@ -11,8 +11,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Parser of import data into command objects.
 
@@ -163,9 +162,11 @@ The grammar is:
 import re
 import sys
 
-import commands
-import dates
-import errors
+from fastimport import (
+    commands,
+    dates,
+    errors,
+    )
 
 
 ## Stream parsing ##
@@ -258,7 +259,7 @@ _WHO_RE = re.compile(r'([^<]*)<(.*)>')
 class ImportParser(LineBasedParser):
 
     def __init__(self, input, verbose=False, output=sys.stdout,
-        user_mapper=None):
+        user_mapper=None, strict=True):
         """A Parser of import commands.
 
         :param input: the file-like object to read from
@@ -266,13 +267,16 @@ class ImportParser(LineBasedParser):
         :param output: the file-like object to write messages to (YAGNI?)
         :param user_mapper: if not None, the UserMapper used to adjust
           user-ids for authors, committers and taggers.
+        :param strict: Raise errors on strictly invalid data
         """
         LineBasedParser.__init__(self, input)
         self.verbose = verbose
         self.output = output
         self.user_mapper = user_mapper
+        self.strict = strict
         # We auto-detect the date format when a date is first encountered
         self.date_parser = None
+        self.features = {}
 
     def warning(self, msg):
         sys.stderr.write("warning line %d: %s\n" % (self.lineno, msg))
@@ -282,6 +286,8 @@ class ImportParser(LineBasedParser):
         while True:
             line = self.next_line()
             if line is None:
+                if 'done' in self.features:
+                    raise errors.PrematureEndOfStream(self.lineno)
                 break
             elif len(line) == 0 or line.startswith('#'):
                 continue
@@ -388,6 +394,7 @@ class ImportParser(LineBasedParser):
             value = self._path(parts[1])
         else:
             value = None
+        self.features[name] = value
         return commands.FeatureCommand(name, value, lineno=self.lineno)
 
     def _parse_file_modify(self, info):
@@ -416,7 +423,8 @@ class ImportParser(LineBasedParser):
     def _parse_tag(self, name):
         """Parse a tag command."""
         from_ = self._get_from('tag')
-        tagger = self._get_user_info('tag', 'tagger', accept_just_who=True)
+        tagger = self._get_user_info('tag', 'tagger',
+                accept_just_who=True)
         message = self._get_data('tag', 'message')
         return commands.TagCommand(name, from_, tagger, message)
 
@@ -519,19 +527,25 @@ class ImportParser(LineBasedParser):
             except ValueError:
                 print "failed to parse datestr '%s'" % (datestr,)
                 raise
+            name = match.group(1)
+            email = match.group(2)
         else:
             match = _WHO_RE.search(s)
             if accept_just_who and match:
                 # HACK around missing time
                 # TODO: output a warning here
                 when = dates.DATE_PARSERS_BY_NAME['now']('now')
-            else:
+                name = match.group(1)
+                email = match.group(2)
+            elif self.strict:
                 self.abort(errors.BadFormat, cmd, section, s)
-        name = match.group(1)
+            else:
+                name = s
+                email = None
+                when = dates.DATE_PARSERS_BY_NAME['now']('now')
         if len(name) > 0:
             if name[-1] == " ":
                 name = name[:-1]
-        email = match.group(2)
         # While it shouldn't happen, some datasets have email addresses
         # which contain unicode characters. See bug 338186. We sanitize
         # the data at this level just in case.
